<schema xmlns="http://www.ascc.net/xml/schematron">

<!-- Checks for the master document: -->
<pattern name="master">

<!-- Presence of page documents and orthogonal sources -->
<rule context="menu//page | blocks/block">
  <report test="
      (for $l in $master//languages/lang 
        return boolean(document(eg:page-src(@id, $l)))) 
      = false()">
    A source document not found for "<value-of select="@id"/>".
  </report>
  <assert test="
      every $i in 
        (for $l in $master//languages/lang 
          return (
            if (@select) 
              then boolean(document(eg:page-src(@id, $l))
                             //block[@id=current()/@select])
              else true()))
        satisfies $i">
    A block with @id="<value-of select="@select"/>" that the
    "<value-of select="@id"/>" orthogonal block refers to
    is missing.
  </assert>
</rule>

</pattern>

<!-- Checks for page documents: -->
<pattern name="page">

<!-- Internal links -->
<rule context="int | link[@linktype='internal']">
  <assert test="
      @link = (for $i in $master//(page|block)/(@id|@alias)
                 return tokenize($i, '\s+'))">
    Broken internal link: no 'page' with 
    @id="<value-of select="@link"/>" in the 
    master document. Valid identifiers are:
    <value-of select="
        string-join(
          (for $i in $master//(page|block)/(@id|@alias)
             return tokenize($i, '\s+')), ', ')"/>.
  </assert>
</rule>

<!-- External links -->
<rule context="ext | link[@linktype='external']">
  <assert test="
      boolean(unparsed-text(eg:ext-link(@link), 'iso-8859-1'))">
    Broken external link: <value-of select="@link"/>.
  </assert>
</rule>

<!-- Language links -->
<rule context="
    lang[not(ancestor::languages)] | link[@linktype='language']">
  <assert test="@link = $master//languages/lang">
    Broken language link: no "<value-of select="@link"/>" language.
  </assert>
</rule>

<!-- Orthogonal references -->
<rule context="page//block[@idref]">
  <assert test="@idref = $master//blocks/block/@id">
    A block with @idref must match the @id of one of the 'block' 
    elements in the master document.
  </assert>
</rule>

<!-- Presence of images -->
<rule context="section[@image]">
  <assert test="
      files:exists(eg:os-path(concat($out-img, @image, '.png')))">
    Image file missing: 
    <value-of select="concat($out-img, @image, '.png')"/>.
  </assert>
</rule>

<!-- Side-wide uniqueness of @id in p and head elements -->
<rule context="p[@id] | head[@id]">
  <assert test="
      count(
        for $src in distinct-values($master//(page|block)/@src) 
          return 
            document(
              concat($src-path, $lang, '/', $src, $src-ext))
              //(p|head)[@id=current()/@id]) 
      = 1">
    Non-unique @id in a 'p' or 'head':
    @id="<value-of select="current()/@id"/>" is used in: 
    <value-of select="
        string-join(
          $master//(page|block)[document(eg:page-src(@id, $lang))
            //(p|head)[@id=current()/@id]]/@src, ', ')"/>.
  </assert>
</rule>

</pattern>
</schema>