<xsl:template name="top-menu"> 
  <!-- Menu layout code... --> 
  <xsl:apply-templates select="$master//menu/item"/> 
  <!-- Menu layout code... -->
</xsl:template>
   
<xsl:template match="menu/item">
  <xsl:variable name="src" select="
      page[1]/@src"/>
  <xsl:variable name="label" select="
      label/translation[@lang=$lang]"/>

  <!-- Composing filename for the graphic menu button: -->
  <xsl:variable name="filename" select="
      concat(
        string(1 + count(preceding-sibling::*)), 
        eg:letters-only($label))"/>

  <!-- Generating the menu button image: -->
  <xsl:if test="$images='yes'">
    <xsl:call-template name="create-image">
      <xsl:with-param name="label" select="
          $label"/>
      <xsl:with-param name="filename" select="
          concat($out-img, $filename)"/>
    </xsl:call-template>
  </xsl:if>

  <!-- Menu item layout code... -->
  <xsl:choose>
    <xsl:when test="$src = $current">
      <!-- Current page is the root of this branch 
           (no link, branch active): -->
      <span class="active">
        <img src="{$target-img}{$filename}.png" alt="{$label}"/>
      </span>
    </xsl:when>

    <xsl:when test="($src != $current) and (page/@src = $current)">
      <!-- Current page is one of the pages (not root) of this branch 
           (link, branch active): -->
      <a href="{$target-path}{$lang}/{$src}{$out-ext}" 
          class="active">
        <img src="{$target-img}{$filename}.png" alt="{$label}"/>
      </a>
    </xsl:when>

    <xsl:when test="$src != $current">
      <!-- Current page does not belong to this branch 
           (link, branch inactive): -->
      <a href="{$target-path}{$lang}/{$src}{$out-ext}" 
          class="inactive">
        <img src="{$target-img}{$filename}.png" alt="{$label}"/>
      </a>
    </xsl:when>
  </xsl:choose> 
  <!-- Menu item layout code... -->
</xsl:template>

<xsl:function name="eg:letters-only">
  <xsl:param name="s"/>
  <xsl:value-of select="
      lower-case(replace($s, '\ |,|\.|!|\?', ''))"/>
</xsl:function>