<xsl:template match="/page"> 
  <xsl:message>
    processing: <xsl:value-of select="$current"/>
  </xsl:message>
  <!-- Credits are in XML comments, 
       so they can come before the root element: -->
  <xsl:call-template name="credits"/>
  <html>
    <head>
      <xsl:apply-templates select="@keywords | @description" 
          mode="meta"/>
      <xsl:call-template name="title"/>
      <xsl:call-template name="css"/>
      <xsl:call-template name="javascript"/>
    </head>
    <body bgcolor="#ffffff" background="{$target-img}bg.gif">

      <!-- Main layout table starts here -->
      <table style="table-layout: fixed; width: 100%;"
          cellpadding="0" cellspacing="0" border="0">
        <xsl:choose>
          <xsl:when test="$frontpage">
            <!-- Static content (logo, etc.) specific to the front page -->
          </xsl:when>
          <xsl:otherwise>
            <!-- Static content specific to a subpage -->
          </xsl:otherwise>
        </xsl:choose>

        <!-- Layout code... -->

        <!-- Calling menu template: -->
        <xsl:call-template name="top-menu"/>

        <!-- More layout code... -->

        <!-- Processing content blocks: -->
        <xsl:apply-templates select="block[not(@idref)]"/>

        <!-- More layout code... -->

        <!-- Processing orthogonal blocks: -->
        <xsl:apply-templates select="block[@idref]"/>

        <!-- Inserting page footer: -->
        <xsl:call-template name="page-footer"/>

        <!-- Closing main layout table: -->
      </table>
    </body>
  </html>

  <!-- Creating another version of the page: -->
  <xsl:result-document format="html"
      href="{$out-path}{$lang}{$current}-print{$out-ext}">
    <xsl:message>creating printer-friendly page</xsl:message>
    <html>
      <head>
        <xsl:call-template name="title"/>
        <xsl:call-template name="css-printable"/>
      </head>
      <body>
        <!-- Inserting a simple page header: -->
        <xsl:call-template name="printable-head"/>
     
        <!-- Processing content blocks: -->
        <xsl:apply-templates select="block[not(@idref)]"/>
    
        <!-- Inserting page footer: -->
        <xsl:call-template name="page-footer"/>
      </body>
    </html>
  </xsl:result-document>

</xsl:template>