<?xml version="1.0" encoding="iso-8859-1"?>

<!DOCTYPE xsl:stylesheet [ 
  <!ENTITY copy "&#169;"> 
  <!ENTITY nbsp "&#160;"> 
  <!ENTITY mdash "&#8212;">
]>

<xsl:stylesheet
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0"
    xmlns:eg="http://www.example.org/"
    xmlns:files="com.projectname.xslt.files"
    xmlns:graph="com.projectname.xslt.graph"
    xmlns:text="com.projectname.xslt.text"
    extension-element-prefixes="eg files graph text">

<xsl:import href="_lib.xsl"/> 

<xsl:output method="html" encoding="US-ASCII" indent="no" 
    doctype-public="-//W3C//DTD HTML 4.0 Transitional//EN"/>
<xsl:output name="txt" method="text" encoding="iso-8859-1" 
    omit-xml-declaration="yes"/>
<xsl:output name="xml" method="xml" encoding="iso-8859-1"
    indent="yes"/>

<!-- Page skeleton -->
<xsl:template match="/page">
  <xsl:message>
    processing: <xsl:value-of select="$current"/>
  </xsl:message>
  <xsl:message>
    lang: <xsl:value-of select="$lang"/> 
    env: <xsl:value-of select="$env"/>
    src-path: <xsl:value-of select="$src-path"/>
    out-path: <xsl:value-of select="$out-path"/>
    target-path: <xsl:value-of select="$target-path"/>
  </xsl:message>
  <!-- Credits are in XML comments, 
       so they can come before the root element: -->
  <xsl:call-template name="credits"/>
  <html>
    <head>
      <xsl:apply-templates select="@keywords | @description"/>
      <xsl:call-template name="title"/>
      <xsl:call-template name="css"/>
    </head>
    <body>
      <table style="table-layout: fixed; width: 100%;">
        <tr>
          <td style="width: 150px;"> 
            <!-- logo cell -->
            <xsl:variable name="logo" select="
                concat($out-img, 'logo.png')"/>
            <img src="{concat($target-img, 'logo.png')}" 
                width="{graph:getw($logo)}" 
                height="{graph:geth($logo)}"
                alt="{$master//html-title/translation[@lang=$lang]}"/>
          </td>
          <td> 
            <!-- top menu cell -->
            <xsl:call-template name="top-menu"/>
          </td>
        </tr>
        <tr>
          <td style="width: 150px; vertical-align: top;"> 
            <!-- side menu cell -->
            <xsl:call-template name="side-menu"/>
          </td>
          <td style="margin: 1em; padding-left: 1em; padding-top: 1em;
              border-left: 2px dotted; border-color: #003399;
              vertical-align: top;"> 
            <!-- body area -->
            <xsl:apply-templates select="block[not(@idref)]"/>
          </td>
        </tr>
        <tr>
          <td style="width: 150px; vertical-align: top;"> 
            <!-- page-footer cell -->
            <xsl:call-template name="page-footer"/>
          </td>
          <td> 
            <!-- orthogonal content area -->
            <table width="100%" style="border-spacing: 1.2em;">
              <tr bgcolor="#ffffff">
                <xsl:apply-templates select="block[@idref]"/>
              </tr>
            </table>
          </td>
        </tr>
      </table>
    </body>
  </html>
</xsl:template>

<!-- Static templates -->

<xsl:template name="credits">
  <xsl:comment>
    XSLT programming by Ed N. Gineer, ed at foobar dot com
  </xsl:comment>
</xsl:template>

<xsl:template match="page/@description">
  <meta name="description" content="{.}"/>
</xsl:template>

<xsl:template match="page/@keywords">
  <meta name="keywords" content="{.}"/>
</xsl:template>

<xsl:template name="title">
  <title> 
    <xsl:value-of select="
        $master//html-title/translation[@lang=$lang]/text()"/>
    <xsl:text>: </xsl:text><xsl:value-of select="title/text()"/>
  </title>
</xsl:template>

<xsl:template name="css">
  <link rel="stylesheet" href="{$target-path}site.css"/>
  <xsl:message>
    creating: site.css
  </xsl:message>
  <xsl:result-document href="file:///{$out-path}site.css" format="txt">
body {font-family: serif; background-color: #ffffff;}
h1 {font-family: sans-serif; font-size: 120%; font-weight: normal;}
h2 {font-size: 100%; font-weight: normal; font-style: italic;}
.inactive {border: 0em; padding-bottom: 4px; border-bottom: 2px dotted;}
.active {border: 0em; padding-bottom: 4px; border: 0px ! important;}
a {text-decoration: none; border-bottom: 2px dotted; color: #003399;}
.footer {font-size: 80%; margin-top: 1em; margin-left: 0.5em;}
.sideitem {text-align: middle; font-size: 70%; padding-top: 2em; 
    margin: 0.5em; text-transform: uppercase;}  
  </xsl:result-document>
</xsl:template>

<!-- Footer stuff -->

<xsl:template name="page-footer">
  <xsl:apply-templates select="$master//page-footer/*"/>
</xsl:template>

<xsl:template match="page-footer/*">
  <p class="footer">
    <xsl:apply-templates select="translation[@lang=$lang]"/>
  </p>
</xsl:template>

<!-- Menus -->

<xsl:template name="side-menu">
  <xsl:apply-templates select="
      $master//menu//item[page/@src = $current]/(page|item)" 
      mode="side"/>
</xsl:template>

<xsl:template name="top-menu">
  <xsl:apply-templates select="$master//menu/item" mode="top"/>
</xsl:template>
   
<xsl:template match="menu/item" mode="top">
  <xsl:variable name="src" select="
      page[1]/@src"/>
  <xsl:variable name="label" select="
      label/translation[@lang=$lang]"/>
  <xsl:variable name="filename" select="
      concat(
        string(1 + count(preceding-sibling::*)), 
        eg:letters-only($label))"/>
  <xsl:if test="$images='yes'">
    <xsl:call-template name="create-image">
      <xsl:with-param name="label" select="$label"/>
      <xsl:with-param name="filename" select="
          concat($out-img, $filename)"/>
    </xsl:call-template>
  </xsl:if>
  <xsl:choose>
    <xsl:when test="$src = $current">
      <span class="active">
        <img src="{$target-img}{$filename}.png" alt="{$label}"
            style="padding: 35px;" 
            width="
              {graph:getw(concat($out-img, $filename, '.png'))}"
            height="
              {graph:geth(concat($out-img, $filename, '.png'))}"/>
      </span>
    </xsl:when>
    <xsl:when test="
        ($src != $current) and 
        (.//page/@src = $current)">
      <a href="
          {$target-path}{$lang}/{$src}{$out-ext}" 
          class="active">
        <img src="{$target-img}{$filename}.png" alt="{$label}" 
            border="0" style="padding: 35px;"
            width="
              {graph:getw(concat($out-img, $filename, '.png'))}"
            height="
              {graph:geth(concat($out-img, $filename, '.png'))}"/>
      </a>
    </xsl:when>
    <xsl:otherwise>
      <a href="
          {$target-path}{$lang}/{$src}{$out-ext}" 
          class="inactive">
        <img src="{$target-img}{$filename}.png" alt="{$label}" 
            border="0" style="padding: 35px;"
            width="
              {graph:getw(concat($out-img, $filename, '.png'))}"
            height="
              {graph:geth(concat($out-img, $filename, '.png'))}"/>
      </a>
    </xsl:otherwise>
  </xsl:choose> 
</xsl:template>

<xsl:template match="menu//item/*" mode="side">
  <xsl:variable name="src" select="
      if (self::item) then page[1]/@src else @src"/>
  <xsl:variable name="label" select="
      if (self::item) 
        then label/translation[@lang=$lang] 
        else document(string(eg:page-src(@id, $lang)))//title"/>
  <xsl:choose>
    <xsl:when test="$src = $current">
      <div class="sideitem"><xsl:value-of select="$label"/></div>
    </xsl:when>
    <xsl:otherwise>
      <div class="sideitem">
        <a href="{$target-path}{$lang}/{$src}{$out-ext}">
          <xsl:value-of select="$label"/>
        </a>
      </div>
    </xsl:otherwise>
  </xsl:choose> 
</xsl:template>

<!-- Blocks and sections -->

<xsl:template match="block[@idref]">
  <xsl:variable name="doc" select="
      document(string(eg:page-src(@idref, $lang)))"/>
  <xsl:variable name="select" select="
      $master//block[@id=current()/@idref]/@select"/>
  <xsl:variable name="from" select="
      if ($select) 
        then $doc//block[@id=$select] 
        else $doc//block[1]"/>

  <td style="margin-left: 1em; vertical-align: top; width: 50%;
      border: 2px dotted; border-color: #003399;">
    <div style="padding: 0.3em;">
      <xsl:apply-templates select="$from//section | $from/p"/>
    </div>
  </td>
</xsl:template>

<xsl:template match="block[not(@idref)]">
  <xsl:apply-templates select="section | p"/>
</xsl:template>

<xsl:template match="section">
  <table style="margin-bottom: 1em;">
    <tr>
      <td style="vertical-align: top;"> 
        <!-- image cell -->
        <xsl:if test="@image">
          <img src="{$target-img}{@image}.png" alt="{head}"/>
        </xsl:if>
      </td>
      <td>  
        <h1><xsl:apply-templates select="head"/></h1>
        <h2><xsl:apply-templates select="subhead"/></h2>
        <xsl:apply-templates select="p"/>
      </td>
    </tr>
  </table>
</xsl:template>

<!-- Image generation -->

<xsl:function name="eg:letters-only">
  <xsl:param name="s"/>
  <!-- remove punctuation, replace German accented letters -->
  <xsl:value-of select="
      translate(
        lower-case(replace($s, '\ |,|\.|!|\?', '')),
        '&#xc4;&#xd6;&#xdc;&#xe4;&#xf6;&#xfc;',
        'aouaou')"/>
</xsl:function>

<xsl:template name="create-image">
  <xsl:param name="label"/>
  <xsl:param name="filename"/>
  <xsl:variable name="svg" select="concat($filename, '.svg')"/>
  <xsl:variable name="png" select="concat($filename, '.png')"/>
  <xsl:result-document href="file:///{$svg}" format="xml">
    <svg width="500px" height="100px" 
        xmlns="http://www.w3.org/2000/svg" 
        xmlns:xlink="http://www.w3.org/1999/xlink">
      <defs>
        <font-face font-family="Pushkin">
          <font-face-src>
            <font-face-uri 
                xlink:href="{$src-path}pushkin.svg#font"/>
          </font-face-src>
        </font-face>
      </defs>

      <text style="
          fill: #000000; font-family: Pushkin; font-size: 30px;" 
          x="10px" y="50px">
        <xsl:value-of select="$label"/>
      </text>
    </svg>
  </xsl:result-document>
  <xsl:message>
    <xsl:value-of select="
        files:run(
          concat(
            'java org.apache.batik.apps.rasterizer.Main ', 
            eg:os-path($svg)))"/>
  </xsl:message>
  <xsl:message>
    <xsl:value-of select="
        files:run(concat('mogrify -trim ', eg:os-path($png)))"/>
  </xsl:message>
</xsl:template>

<!-- Text markup -->

<xsl:template match="p">
  <p><xsl:apply-templates/></p>
</xsl:template>

<xsl:template match="em">
  <em><xsl:apply-templates/></em>
</xsl:template>

<!-- Links -->

<xsl:template match="link[@linktype='internal'] | int">
  <a href="{eg:page-link($target-path, @link, $lang)}">
    <xsl:apply-templates/>
  </a>
</xsl:template>

<xsl:template match="link[@linktype='external'] | ext">
  <a href="{eg:ext-link(@link)}">
    <xsl:apply-templates/>
  </a>
</xsl:template>

<xsl:template match="link[@linktype='mailto'] | mailto">
  <a href="mailto:{@link}">
    <xsl:apply-templates/>
  </a>
</xsl:template>

<xsl:template match="link[@linktype='lang'] | lang">
  <a href="{$target-path}{@link}/{$current}{$out-ext}">
    <xsl:apply-templates/>
  </a>
</xsl:template>

<!-- Typography -->

<xsl:template match="p//text() | head//text() | subhead//text()">
  <xsl:value-of select="text:typography(.)"/>
</xsl:template>

<!-- Batch processing -->

<xsl:template match="/site">
  <xsl:result-document 
      href="file:///{eg:os-path($src-path)}validate-commands.log" 
      format="txt">
    <xsl:apply-templates select="menu//page" mode="validate"/>
  </xsl:result-document>
  <xsl:result-document 
      href="file:///{eg:os-path($src-path)}transform-commands.log" 
      format="txt">
    <xsl:apply-templates select="menu//page" mode="transform"/>
  </xsl:result-document>
</xsl:template>

<xsl:template match="page" mode="validate">
  <xsl:variable name="id" select="@id"/>
  <xsl:for-each select="$master//languages/lang">
    <xsl:message>
      Validating <xsl:value-of select="eg:page-src($id, .)"/>
    </xsl:message>
    <xsl:value-of select="
        files:run(concat(
          'java net.sf.saxon.Transform -l ', 
          eg:os-path(eg:page-src($id, .)),
          ' schema-compiled.xsl',
          ' env=', $env
        ))"/>
  </xsl:for-each>
</xsl:template>

<xsl:template match="page" mode="transform">
  <xsl:variable name="id" select="@id"/>
  <xsl:for-each select="$master//languages/lang">
    <xsl:message>
      Transforming <xsl:value-of select="eg:page-src($id, .)"/>
    </xsl:message>
    <xsl:value-of select="
        files:run(concat(
          'java net.sf.saxon.Transform ', 
          '-o ', eg:os-path(eg:page-link($out-path, $id, .)),
          ' ', eg:os-path(eg:page-src($id, .)),
          ' style.xsl',
          ' env=', $env,
          ' images=', if ($id='home') then $images else 'no'
        ))"/>
  </xsl:for-each>
</xsl:template>

</xsl:stylesheet>